%
%    This is the driver routine of software package for computing a partial
%    elastic registration of two simple surfaces in 3-d space using Dynamic
%    programming and the corresponding elastic shape distance between them.
%
%*******************************************************************
%
%    Author: Javier Bernal
%            National Institute of Standards and Technology (NIST)
%
%     Disclaimer:
%
%     This software was developed at the National Institute of Standards
%     and Technology by employees of the Federal Government in the
%     course of their official duties. Pursuant to title 17 Section 105
%     of the United States Code this software is not subject to
%     copyright protection and is in the public domain. This software is
%     experimental. NIST assumes no responsibility whatsoever for its
%     use by other parties, and makes no guarantees, expressed or
%     implied, about its quality, reliability, or any other
%     characteristic. We would appreciate acknowledgement if the
%     software is used.
%
clear;
%
tic;
%fprintf('\n \n');
%
% Driver routine for computing partial elastic registration of two simple
% surfaces in 3-d space using Dynamic programming and the corresponding
% elastic shape distance between them.
%
% Each surface is an image of the unit square of the plane and is given as
% N rows of points on the surface, each row of length M, the rows on the
% surface corresponding to rows in the unit square of the plane (from left
% to right, from bottom to top).
%
iten = 7;
% iten = maximum number of while loop iterations allowed in the main procedure
%fprintf('\n \n');
%
both_drct = 0;
% both_drct is for determining whether the first surface is to be
% considered in the reversed direction of rows as well (0 if no, 1 if yes).
% Here the reversed direction of rows means that the N rows on the surface
% are reversed: the 1st row becomes the Nth row, and viceversa, etc.
% Accordingly, it is assumed that the surfaces are oriented in such a
% way that only the two directions addressed in this program are possible.
% (domain partition in y direction should be uniform if yes)
% Actually eight directions are possible, two per corner of the unit square,
% but here for the sake of simplicity we assume only two are possible as 
% pointed out above.
%
if (both_drct)
   fprintf('\n First surface considered in two directions of rows \n');
   fprintf('\n Domain partition in y direction should be uniform \n')
%   fprintf('\n \n');
else
   fprintf('\n First surface considered in given direction of rows only \n');
%   fprintf('\n \n');
end
%
% R = optimal rotation

% read x domain partition of surfaces 
d1 = fopen('x_domain.txt');
r1 = fscanf(d1, '%g ', [1 inf]);
fclose(d1);

M = length(r1);

if M < 3
   error('Not enough columns in surfaces data');
end

% read y domain partition of surfaces
d3 = fopen('y_domain.txt');
r3 = fscanf(d3, '%g ', [1 inf]);
fclose(d3);

N = length(r3);

if N < 3
   error('Not enough rows in surfaces data');
end

% read surface 1 data
f1 = fopen('surface1.txt');
b1 = fscanf(f1, '%g %g %g', [3 inf]);
fclose(f1);
 
% read surface 2 data
f2 = fopen('surface2.txt');
b2 = fscanf(f2, '%g %g %g', [3 inf]);
fclose(f2);
 
N1 = length(b1);
N2 = length(b2);

MN = M*N;

if N1 ~= MN
   error('Size of surface 1 does not match domain sizes');
end


if N2 ~= MN
   error('Size of surface 2 does not match domain sizes');
end
%min(r1)
%max(r1)
%min(r3)
%max(r3)
%scale r and t
   dt = max(r1)-min(r1);
   r = (r1-min(r1))/dt;
   r(1) = 0;
   r(M) = 1;
%
   dt = max(r3)-min(r3);
   t = (r3-min(r3))/dt;
   t(1) = 0;
   t(N) = 1;
%
%f1=b1;
%f2=b2;
%
%imath=0;
%if(imath)
%   ispan=25
%   tic;
%   f1 = smooth(s1,b1,ispan);
%   f2 = smooth(s2,b2,ispan);
%   f1=f1';
%   f2=f2';
%   timing_smooth = toc;
%   timing_smooth
%end
%
   [f1opt,f2opt,f1,f2,gammaopt,iter,minE,drct_srfc,Ropt] = ESD_comp_surf_3d(iten,both_drct,b1,b2,r,t,MN,M,N);
%pause
%
timing = toc

f3 = fopen('f1_data.txt', 'w');
fprintf(f3,'%12.8f %12.8f %12.8f \n',f1);
fclose(f3);

f3 = fopen('f2_data.txt', 'w');
fprintf(f3,'%12.8f %12.8f %12.8f \n',f2);
fclose(f3);

f3 = fopen('gammaopt_data.txt', 'w');
fprintf(f3,'%12.8f \n',gammaopt);
fclose(f3);

f3 = fopen('f1opt_data.txt', 'w');
fprintf(f3,'%12.8f %12.8f %12.8f \n',f1opt);
fclose(f3);

f3 = fopen('f2opt_data.txt', 'w');
fprintf(f3,'%12.8f %12.8f %12.8f \n',f2opt);
fclose(f3);

f3 = fopen('ropt_data.txt', 'w');
fprintf(f3,'%5.5f %5.5f %5.5f \n', Ropt(1,1), Ropt(1,2), Ropt(1,3));
fprintf(f3,'%5.5f %5.5f %5.5f \n', Ropt(2,1), Ropt(2,2), Ropt(2,3));
fprintf(f3,'%5.5f %5.5f %5.5f \n', Ropt(3,1), Ropt(3,2), Ropt(3,3));
fclose(f3);

%figure;
%plot(t,gammabest,'b-');
%
%fprintf('\n \n');
if (drct_srfc == 1)
   fprintf('\n Direction of first surface for optimality is as given \n');
%   fprintf('\n \n');
else
   fprintf('\n Direction of first surface for optimality is reversed \n');
%   fprintf('\n \n');
end
%fprintf('\n \n');
%fprintf('\n Execution time = %5.5f \n', timing);
%fprintf('\n \n');
fprintf('\n Computed distance = %5.9f \n', minE);
%fprintf('\n \n');
fprintf('\n Number of while loop iterations and maximum number allowed = %2.0f %2.0f \n',iter,iten);
%fprintf('\n \n');
fprintf('\n Computed optimal matrix = \n');
fprintf('\n %5.5f %5.5f %5.5f \n', Ropt(1,1), Ropt(1,2), Ropt(1,3));
fprintf('\n %5.5f %5.5f %5.5f \n', Ropt(2,1), Ropt(2,2), Ropt(2,3));
fprintf('\n %5.5f %5.5f %5.5f \n', Ropt(3,1), Ropt(3,2), Ropt(3,3));
fprintf('\n \n');
%
